---
title: Ruby 2.5 changes
prev: 2.6
description: Ruby 2.5 full and annotated changelog
---

# Ruby 2.5

<!--
* **Details:**
* **Reason:**
* **Discussion:** [Feature #](https://bugs.ruby-lang.org/issues/)
* **Documentation:** []()
* **Code:**
* **Follow-up:**
 -->

* **Released at:** Dec 25, 2017 (<a class="github" href="https://github.com/ruby/ruby/blob/trunk/doc/NEWS-2.5.0">NEWS</a> file)
* **Status (as of Feb 07, 2023):** EOL, latest is 2.5.9
* **This document first published:** Jun 6, 2019
* **Last change to this document:** Feb 07, 2023

## Highlights[](#highlights)

* [`rescue`/`ensure` inside blocks](#rescueelseensure-are-allowed-inside-blocks)
* [`#yield_self`](#yield_self)
* [`Struct` with keyword initialization](#struct-with-keyword-arguments)
* [Exceptions output changed](#backtrace-and-error-message-in-reverse-order)

## Language[](#language)

### Top-level constant look-up is removed.[](#top-level-constant-look-up-is-removed)

`Foo::Bar` doesn't fallback to `::Bar` anymore, when `Foo::Bar` is not found

* **Reason:** The behavior was considered to be bringing more problems and bugs than benefits for a long time, and always issued a warning.
* **Discussion:** <a class="tracker feature" href="https://bugs.ruby-lang.org/issues/11547">Feature #11547</a>
* **Code:**
  ```ruby
  Hash::String
  # Ruby 2.4:
  #   (irb):1: warning: toplevel constant String referenced by Hash::String
  #    => String
  # Ruby 2.5:
  #   NameError (uninitialized constant Hash::String)
  ```

### `rescue`/`else`/`ensure` are allowed inside blocks[](#rescueelseensure-are-allowed-inside-blocks)

* **Reason:** for consistency with other `something` / `end` constructs (class and method bodies), which allowed `rescue` without wrapping in additional `begin`/`end`
* **Discussion:** <a class="tracker feature" href="https://bugs.ruby-lang.org/issues/12906">Feature #12906</a>
* **Code:**
  ```ruby
  (0..5).map do |val|
    puts 100 / val
  rescue ZeroDivisionError
    puts '<undividable>'
  end
  # prints:
  #  <undividable>
  #  100
  #  50
  #  33
  #  25
  #  20
  ```
* **Notice:** The syntax is not available for `{}` blocks, even multiline ones.

### Refinements work in string interpolations[](#refinements-work-in-string-interpolations)

* **Reason:** Step in a consistent effort to make refinements more useful and consistent, working everywhere regular methods work
* **Discussion:** <a class="tracker feature" href="https://bugs.ruby-lang.org/issues/13812">Feature #13812</a> (Japanese)
* **Code:**
  ```ruby
  module Ref
    refine Time do
      def to_s
        "#{hour}:#{min} at #{month}/#{day}/#{year}"
      end
    end
  end

  using Ref

  puts "Now is #{Time.now}"
  # Ruby 2.4: "Now is 2019-05-27 17:49:27 +0300" -- default to_s
  # Ruby 2.5: "Now is 17:51 at 5/27/2019" -- refined to_s
  ```
* **Follow-up:** Ruby 2.6 added [more features](https://rubyreferences.github.io/rubychanges/2.6.html#refinements-improved-visibility) making refinements available in various contexts.

## Core classes and modules[](#core-classes-and-modules)

### `Kernel`[](#kernel)

_Note that methods defined in `Kernel` are typically available on any object, that's why these changes are important._

#### `#yield_self`[](#yield_self)

* **Reason:** Chaining singular value processing the same way `Enumerable` allows chaining of sequence processing methods.
* **Discussion:** <a class="tracker feature" href="https://bugs.ruby-lang.org/issues/6721">Feature #6721</a>
* **Code:**
  ```ruby
  [BASE_URL, path].join('/')
      .yield_self { |url| open(url).read }
      .yield_self { |body| JSON.parse(body, symbolyze_names: true) }
      .dig(:data, :items)
      .yield_self { |items| File.write('response.yml', items.to_yaml) }
  ```
* **Follow-up:** Ruby 2.6 [renamed (aliased)](https://rubyreferences.github.io/rubychanges/2.6.html#then-as-an-alias-for-yield_self) unfortunate method name to `#then`

#### `#pp`[](#pp)

`pp` standard library is now automatically required on the first call to `Kernel#pp`

* **Discussion:** <a class="tracker feature" href="https://bugs.ruby-lang.org/issues/14123">Feature #14123</a>

#### `#warn`: `uplevel:` keyword argument[](#warn-uplevel-keyword-argument)

When `uplevel: N` provided to `Kernel#warn`, the warning message includes the file and line of the call `N` steps up in the callstack.

* **Reason:** It is sometimes hard to understand exactly what code caused a warning. This feature helps identify the offending caller.
* **Discussion:** <a class="tracker feature" href="https://bugs.ruby-lang.org/issues/12882">Feature #12882</a>
* **Documentation:** <a class="ruby-doc" href="https://ruby-doc.org/core-2.6/Kernel.html#method-i-warn"><code>Kernel#warn</code></a> (it became somewhat documented at Ruby 2.6, but the feature itself has been available since 2.5)
* **Code:**
  ```ruby
  def that_bad_function
    warn('do not call me')
    warn('I am talking to YOU', uplevel: 1)
    warn('and tell all your clients!', uplevel: 2)
  end

  def try_using_it
    that_bad_function # line 8
  end

  try_using_it # line 11

  # Prints:
  #  do not call me
  #  test.rb:8: warning: I am talking to YOU
  #  test.rb:11: warning: and tell all your clients!
  ```

#### `#warn`: call `Warning.warn`[](#warn-call-warningwarn)

Ruby 2.4 introduced redefinable `Warning.warn` method, which allowed to control how warnings are handled. But `warn` method wasn't calling it, so initially only warnings issued from C code could've been handled. 2.5 fixes it. <small>(Was not mentioned in NEWS-2.5.0)</small>

* **Discussion:** <a class="tracker feature" href="https://bugs.ruby-lang.org/issues/12299#note-14#note-14">Feature #12299<small>#note-14</small></a>
* **Documentation:** —
* **Code:**
  ```ruby
  def Warning.warn(msg)
    puts ".warn called with: #{msg.inspect}"
  end

  warn 'foo', 'bar'
  # Ruby 2.4 prints:
  #  foo
  #  bar
  # Ruby 2.5 prints:
  #  .warn called with: "foo\nbar\n"
  ```
* **Notes:** `Kernel#warn` accepts multiple arguments, and joins them with `"\n"` to pass to `Warning#warn` as a single string.

### `Module`: methods for defining methods and accessors became public[](#module-methods-for-defining-methods-and-accessors-became-public)

* **Reason:** The methods affected are frequently used in metaprogramming, which prior to 2.5 required reopening classes/modules or using `send`
* **Discussion:** <a class="tracker feature" href="https://bugs.ruby-lang.org/issues/14132">Feature #14132</a>, <a class="tracker feature" href="https://bugs.ruby-lang.org/issues/14133">Feature #14133</a>
* **Affected methods:** `#attr`, `#attr_accessor`, `#attr_reader` `#attr_writer`, `#define_method`, `#alias_method`, `#undef_method`, `#remove_method`.
* **Code:**
  ```ruby
  Array.define_method(:half_size) { size.to_f / 2 }
  # => :half_size
  [1, 2, 3].half_size
  # => 1.5
  ```

### `Method#===`[](#method)

* **Reason:** `Method` object can be used in a lot of contexts where the `Proc` can be used, and this one context (case equality) was lacking.
* **Discussion:** <a class="tracker feature" href="https://bugs.ruby-lang.org/issues/14142">Feature #14142</a>
* **Documentation:** <a class="ruby-doc" href="https://ruby-doc.org/core-2.5.0/Method.html#method-i-3D-3D-3D"><code>Method#===</code></a>
* **Code:**
  ```ruby
  require 'prime'
  case 137
  when Prime.method(:prime?)
    puts 'prime'
  else
    puts 'not!'
  end
  # prints 'prime' in Ruby 2.5+, 'not' in Ruby 2.4
  ```

### Integer[](#integer)

#### `#pow`: modulo argument[](#pow-modulo-argument)

* **Reason:** [Modular exponentiation](https://en.wikipedia.org/wiki/Modular_exponentiation) is a useful concept, especially in cryptography, and can be vastly ineffective if performed as to sequential operations.
* **Discussion:** <a class="tracker feature" href="https://bugs.ruby-lang.org/issues/12508">Feature #12508</a>, <a class="tracker feature" href="https://bugs.ruby-lang.org/issues/11003">Feature #11003</a>
* **Documentation:** <a class="ruby-doc" href="https://ruby-doc.org/core-2.5.0/Integer.html#method-i-pow"><code>Integer#pow</code></a>
* **Code:**
  ```ruby
  2120078484650058507891187874713297895455.
    pow(5478118174010360425845660566650432540723,
        5263488859030795548286226023720904036518)
  # => 4481650795473624846969600733813414725093
  ```

#### `#allbits?`, `#anybits?`, `#nobits?`[](#allbits-anybits-nobits)

* **Reason:** It is noted that testing values against bitmasks in Ruby is _a bit_ non-atomic: first, calculate `value & flags`, and then compare result with zero (no common flags / at least one common flag), or original value (all common flags); the new methods should make the tests clearer
* **Discussion:** <a class="tracker feature" href="https://bugs.ruby-lang.org/issues/12753">Feature #12753</a>
* **Documentation:** <a class="ruby-doc" href="https://ruby-doc.org/core-2.5.0/Integer.html#method-i-allbits-3F"><code>Integer#allbits?</code></a>, <a class="ruby-doc" href="https://ruby-doc.org/core-2.5.0/Integer.html#method-i-allbits-3F"><code>Integer#anybits?</code></a>, <a class="ruby-doc" href="https://ruby-doc.org/core-2.5.0/Integer.html#method-i-allbits-3F"><code>Integer#nobits?</code></a>
* **Code:**
  ```ruby
  CARNIVOROUS = 0b001
  MAMMAL = 0b010
  TERRASTRIAL = 0b100
  lizard = 0b101
  lizard.allbits?(CARNIVOROUS | MAMMAL) # => false
  lizard.anybits?(CARNIVOROUS | MAMMAL) # => true
  lizard.nobits?(CARNIVOROUS | MAMMAL) # => false
  ```

#### `.sqrt`[](#sqrt)

* **Reason:** `Math.sqrt(x).to_i` for large numbers loses precision, so `Integer.sqrt` implements effective and precise _integer_ square root algorithm
* **Discussion:** <a class="tracker feature" href="https://bugs.ruby-lang.org/issues/13219">Feature #13219</a>
* **Documentation:** <a class="ruby-doc" href="https://ruby-doc.org/core-2.5.0/Integer.html#method-c-sqrt"><code>Integer.sqrt</code></a>
* **Code:**
  ```ruby
  Integer.sqrt(10**33)
  # => 31622776601683793
  Integer.sqrt(10**33)**2
  # => 999999999999999979762122758866849
  Math.sqrt(10**33).to_i
  # => 31622776601683792
  Math.sqrt(10**33).to_i**2
  # => 999999999999999916516569555499264
  ```

### `String`[](#string)

#### `#delete_prefix`, `#delete_prefix!`, `#delete_suffix`, `#delete_suffix!`[](#delete_prefix-delete_prefix-delete_suffix-delete_suffix)

* **Discussion:** <a class="tracker feature" href="https://bugs.ruby-lang.org/issues/12694">Feature #12694</a>, <a class="tracker feature" href="https://bugs.ruby-lang.org/issues/13665">Feature #13665</a>
* **Documentation:** <a class="ruby-doc" href="https://ruby-doc.org/core-2.5.0/String.html#method-i-delete_prefix"><code>String#delete_prefix</code></a>, <a class="ruby-doc" href="https://ruby-doc.org/core-2.5.0/String.html#method-i-delete_suffix"><code>String#delete_suffix</code></a>
* **Code:**
  ```ruby
  'foo'.delete_prefix('fo') # => 'o'
  'bar'.delete_suffix('r') # => 'ba'

  # Notice that bang-meethods return nil if they've deleted nothing:
  'foo'.delete_prefix!('ba') # => nil
  # This allows to use the methods in ifs:
  descr = RUBY_DESCRIPTION.dup # => "ruby 2.5.1p57 (2018-03-29 revision 63029) [x86_64-linux]"
  if descr.delete_prefix!('ruby ')
    puts "We are running on MRI #{descr}" # => "We are running on MRI 2.5.1p57 ...."
  end
  ```

#### `#each_grapheme_cluster` and `#grapheme_clusters`[](#each_grapheme_cluster-and-grapheme_clusters)

* **Reason:** In modern Unicode, lots of on-screen characters (graphemes) are represented by several characters, which are logically combined into "grapheme clusters"; now Ruby has a proper way to work with those concepts.
* **Discussion:** <a class="tracker feature" href="https://bugs.ruby-lang.org/issues/13780">Feature #13780</a>
* **Documentation:** <a class="ruby-doc" href="https://ruby-doc.org/core-2.5.0/String.html#method-i-each_grapheme_cluster"><code>String#each_grapheme_cluster</code></a>, <a class="ruby-doc" href="https://ruby-doc.org/core-2.5.0/String.html#method-i-grapheme_clusters"><code>String#grapheme_clusters</code></a>
* **Code:**
  ```ruby
  "🏳️‍🌈".length
  # => 4
  "🏳️‍🌈".chars
  # => ["🏳", "️", "‍", "🌈"]
  "🏳️‍🌈".grapheme_clusters
  # => ["🏳️‍🌈"]
  "🏳️‍🌈".each_grapheme_cluster.map { |s| CGI.escape(s) }
  # => ["%F0%9F%8F%B3%EF%B8%8F%E2%80%8D%F0%9F%8C%88"]
  ```

#### `#undump`[](#undump)

The inverse of <a class="ruby-doc" href="https://ruby-doc.org/core-2.5.0/String.html#method-i-dump"><code>String#dump</code></a> which escapes all non-printable and special characters.

* **Discussion:** <a class="tracker feature" href="https://bugs.ruby-lang.org/issues/12275">Feature #12275</a>
* **Documentation:** <a class="ruby-doc" href="https://ruby-doc.org/core-2.5.0/String.html#method-i-undump"><code>String#undump</code></a>
* **Code:**
  ```ruby
  p <<~STR
    This is "fine".
    Or is it?
  STR
  # Prints "This is \"fine\".\nOr is it?\n"
  # ^ Now, this is dumped string, including quotes around.
  # How to get back to "nice" string?
  puts '"This is \"fine\".\nOr is it?\n"'.undump
  # Prints
  #  This is "fine".
  #  Or is it?
  ```

#### `#casecmp` and `#casecmp?` return `nil` for non-string arguments[](#casecmp-and-casecmp-return-nil-for-non-string-arguments)

Previously, those methods raised `TypeError` on arguments that can't be coerced to `String`, now they just return `nil`

* **Reason:** The behavior made consistent with `==` and `<=>`, as the methods are case-agnostic versions of those operators: when you compare incomparable things, you just get no result at all.
* **Discussion:** <a class="tracker feature" href="https://bugs.ruby-lang.org/issues/13312">Feature #13312</a>
* **Documentation:** <a class="ruby-doc" href="https://ruby-doc.org/core-2.5.0/String.html#method-i-casecmp"><code>String#casecmp</code></a>, <a class="ruby-doc" href="https://ruby-doc.org/core-2.5.0/String.html#method-i-casecmp-3F"><code>String#casecmp?</code></a>
* **Code:**
  ```ruby
  "Foo".casecmp?(:foo)
  # Ruby 2.4: TypeError: no implicit conversion of Symbol into String
  # Ruby 2.5: nil
  ```

#### `#start_with?` accepts a regexp[](#start_with-accepts-a-regexp)

`string.start_with?(/foo/)` is an equivalent of `string.match?(/\Afoo/)`

* **Discussion:** <a class="tracker feature" href="https://bugs.ruby-lang.org/issues/13712">Feature #13712</a>
* **Documentation:** <a class="ruby-doc" href="https://ruby-doc.org/core-2.6/String.html#method-i-start_with-3F"><code>String#start_with?</code></a> (introduced in 2.5, but documented in 2.6)
* **Note:** While looking a symmetric method, `#end_with?` does **not** support regexp; the possibility was discussed in the feature proposal and it was general agreement that the behavior should be symmetric, but eventually the implementation turned out to be too hard.

#### `String#-@` optimized for memory preserving[](#string--optimized-for-memory-preserving)

`String#-` (freezing the string) is optimized to deduplicate frozen object

* **Discussion:** <a class="tracker feature" href="https://bugs.ruby-lang.org/issues/13077">Feature #13077</a>, <a class="tracker feature" href="https://bugs.ruby-lang.org/issues/13295">Feature #13295</a>
* **Documentation:** <a class="ruby-doc" href="https://ruby-doc.org/core-2.5.0/String.html#method-i-2D-40"><code>String#-@</code></a>
* **Code:**
  ```ruby
  str = String.new('not frozen')
  10.times.map { -str }.map(&:object_id)
  # Ruby 2.4: 10 different ids => 10 different frozen string objects
  # Ruby 2.5: 10 same ids => always the same frozen object

  5.times.map { -'test' }.map(&:object_id)
  # Ruby 2.4: 5 different ids => 5 different frozen string objects
  # Ruby 2.5: 5 same ids => always the same frozen object
  ```

### `Regexp`: absence operator[](#regexp-absence-operator)

The underlying <a class="github" href="https://github.com/k-takata/Onigmo">Onigmo</a> Regexp library was updated, which brought new operator: `(?~foo)` means "match any string that **do not** ends with `foo`"

* **Documentation:** —
* **Code:**
  ```ruby
  # Classic example of usefulness: matching C comments:
  code = <<~C
    /* This is a comment.
    It ends here: */
    // And not here: */
  C
  code.scan(%r{/\*.+\*/}m)
  # => ["/* This is a comment.\nIt ends here: */\n// And not here: */"]
  code.scan(%r{/\*(?~\*/)\*/})
  # => ["/* This is a comment.\nIt ends here: */"]
  ```

### `Struct` with keyword arguments[](#struct-with-keyword-arguments)

`Struct` can be created with `keyword_init: true`, and then its instance will require keyword arguments to initialize

* **Discussion:** <a class="tracker feature" href="https://bugs.ruby-lang.org/issues/11925">Feature #11925</a>
* **Documentation:** <a class="ruby-doc" href="https://ruby-doc.org/core-2.5.0/Struct.html#method-c-new"><code>Struct.new</code></a>
* **Code:**
  ```ruby
  Person = Struct.new(:first, :last, :age)
  ModernPerson = Struct.new(:first, :last, :age, keyword_init: true)

  Person.new('Bob', 'Jones', 40)
  # => #<struct Person first="Bob", last="Jones", age=40>
  ModernPerson.new(first: 'Meredith', last: 'Williams', age: 28)
  # => #<struct ModernPerson first="Meredith", last="Williams", age=28>
  ModernPerson.new('Meredith', 'Williams', 28)
  # ArgumentError (wrong number of arguments (given 3, expected 0))
  ```
* **Follow-ups:**
  * Ruby 3.1 introduced [a method](3.1.html#structclasskeyword_init) to check whether some struct should be initialized with keyword arguments, and [a warning](3.1.html#warning-on-passing-keywords-to-a-non-keyword-initialized-struct) on erroneous initialization of non-keyword-args struct with a hash.
  * Ruby 3.2 [allowed](3.2.html#struct-can-be-initialized-by-keyword-arguments-by-default) all structs without explicit `keyword_init:` parameter specified to be initialized by both positional and keyword args.

### `Time.at`: units[](#timeat-units)

`Time.at` second argument was meant to be microseconds, but in 2.5+ it can be micro-, milli- or nano-seconds, specified by third argument: `:millisecond`, `:usec` or `:microsecond` (default), `:nanosecond`

* **Reason:** It was noted that when you have nanoseconds value (available as `Time#nsec`) and want to pass it to `.at`, it is inconvenient to divide it by `1000`.
* **Discussion:** <a class="tracker feature" href="https://bugs.ruby-lang.org/issues/13919">Feature #13919</a>
* **Documentation:** <a class="ruby-doc" href="https://ruby-doc.org/core-2.5.0/Time.html#method-c-at"><code>Time.at</code></a>
* **Code:**
  ```ruby
  nanoseconds = 123456789
  # The only way previously to create time with nsec value
  Time.at(946684800, nanoseconds / 1000.0).nsec # => 123456789
  # New way
  Time.at(946684800, nanoseconds, :nanosecond).nsec # => 123456789
  # Also convenient for some contexts: .123 seconds
  Time.at(946684800, 123, :millisecond).usec
  # => 123000
  ```

### Collections[](#collections)

#### `Enumerable#any?`, `#all?`, `#none?`, and `#one?` accept patterns[](#enumerableany-all-none-and-one-accept-patterns)

Any argument that responds to `#===` may now be passed (akin to `grep`)

* **Discussion:** <a class="tracker feature" href="https://bugs.ruby-lang.org/issues/11286">Feature #11286</a>
* **Documentation:** <a class="ruby-doc" href="https://ruby-doc.org/core-2.5.0/Enumerable.html#method-i-all-3F"><code>Enumerable#all?</code></a>, <a class="ruby-doc" href="https://ruby-doc.org/core-2.5.0/Enumerable.html#method-i-any-3F"><code>Enumerable#any?</code></a>, <a class="ruby-doc" href="https://ruby-doc.org/core-2.5.0/Enumerable.html#method-i-none-3F"><code>Enumerable#none?</code></a>, <a class="ruby-doc" href="https://ruby-doc.org/core-2.5.0/Enumerable.html#method-i-one-3F"><code>Enumerable#one?</code></a>
* **Code:**
  ```ruby
  [1, 2, nil].all?(Numeric) # => false
  ['America', 'Europe', 'Africa', 'Asia'].any?(/^E/) # => true
  (1..20).one?(20..30) # => true
  require 'set'
  [:foo, :bar, :baz].none?(Set[:foo, :test]) # => false
  ```

#### `Array#append` and `#prepend`[](#arrayappend-and-prepend)

Just aliases for `#push` and `#unshift`

* **Reason:** The old methods are symmetric with `#pop` and `#shift`, respectively, but asymmetric with each other and (especially `#unshift`) hard to guess and remember
* **Discussion:** <a class="tracker feature" href="https://bugs.ruby-lang.org/issues/12746">Feature #12746</a>
* **Documentation:** <a class="ruby-doc" href="https://ruby-doc.org/core-2.5.0/Array.html#method-i-append"><code>Array#append</code></a>, <a class="ruby-doc" href="https://ruby-doc.org/core-2.5.0/Array.html#method-i-prepend"><code>Array#prepend</code></a>
* **Code:**
  ```ruby
  ary = [1, 2, 3]
  ary.append(4)
  # => [1, 2, 3, 4]
  ary.prepend(0)
  # => [0, 1, 2, 3, 4]
  ary
  # => [0, 1, 2, 3, 4]
  ```

#### `Hash#transform_keys` and `#transform_keys!`[](#hashtransform_keys-and-transform_keys)

* **Discussion:** <a class="tracker feature" href="https://bugs.ruby-lang.org/issues/13583">Feature #13583</a>
* **Documentation:** <a class="ruby-doc" href="https://ruby-doc.org/core-2.5.0/Hash.html#method-i-transform_keys"><code>Hash#transform_keys</code></a>, <a class="ruby-doc" href="https://ruby-doc.org/core-2.5.0/Hash.html#method-i-transform_keys-21"><code>Hash#transform_keys!</code></a>
* **Code:**
  ```ruby
  {a: 1, b: 2}.transform_keys { |sym| sym.to_s.capitalize }
  # => {"A"=>1, "B"=>2}
  {a: 1, b: 2}.transform_keys(&:to_s)
  # => {"a"=>1, "b"=>2}
  {a: 1, b: 2}.transform_keys(&:length) # non-unique - the last value is taken
  # => {1=>2}
  {a: 1, b: 2}.transform_keys           # without block - returns enumerator
  # => #<Enumerator: {:a=>1, :b=>2}:transform_keys>
  # The latter can be useful when modified by .with_index
  {a: 1, b: 2}.transform_keys.with_index(1) { |k, i| "#{i}: #{k}" }
  # => {"1: a"=>1, "2: b"=>2}
  ```
* **Follow-up:** Since **Ruby 2.5.1**, the behavior of `transform_keys!` (bang version) is changed: Ruby 2.5.0 changes the hash as it goes (the same behavior as the old ActiveSupport method of the same name), whereas Ruby 2.5.1 first calculates the new hash and then replaces the old one:
  ```ruby
  # Ruby 2.5.0, Ruby 2.4 with ActiveSupport
  h = {1 => :hello, 2 => 'world'}
  h.transform_keys!(&:succ) # first, 2 => :hello replaces 2 => 'world', then the same key is processed again
  # => {3=>:hello}

  # Ruby 2.5.1
  h = {1 => :hello, 2 => 'world'}
  h.transform_keys!(&:succ)
  # => {2=>:hello, 3=>"world"}
  ```
  Discussion: <a class="tracker bug" href="https://bugs.ruby-lang.org/issues/14380">Bug #14380</a>
* **Follow-up:** In Ruby 3.0, a new form `transform_keys(from_name: :to_name)` [was introduced](3.0.html#hashtransform_keys-argument-for-key-renaming).

#### `Hash#slice`[](#hashslice)

* **Discussion:** <a class="tracker feature" href="https://bugs.ruby-lang.org/issues/13563">Feature #13563</a>
* **Documentation:** <a class="ruby-doc" href="https://ruby-doc.org/core-2.5.0/Hash.html#method-i-slice"><code>Hash#slice</code></a>
* **Code:**
  ```ruby
  h = { a: 100, b: 200, c: 300 }
  h.slice(:a)           #=> {:a=>100}
  h.slice(:b, :c, :d)   #=> {:b=>200, :c=>300}
  ```
* **Follow-up:** Funnily enough, what started at [#8499](https://bugs.ruby-lang.org/issues/8499) as discussing of importing `#slice`/`#slice!`/`#except` and `#except!` from ActiveSupport, ended up in a several tickets: `#slice` (current one, accepted for 2.5), `#slice!` ([#15863](https://bugs.ruby-lang.org/issues/15863), rejected by Matz, "no real-life use-case"), and `#except` ([#15822](https://bugs.ruby-lang.org/issues/15822), recent ticket, not merged as of May 2019). **UPD:** Finally [made its way](3.0.html#hashexcept) to Ruby 3.0.

### Exceptions[](#exceptions)

#### Backtrace and error message in reverse order[](#backtrace-and-error-message-in-reverse-order)

New behavior is activated if `STDERR` is TTY

* **Discussion:** <a class="tracker feature" href="https://bugs.ruby-lang.org/issues/8661">Feature #8661</a>
* **Code:**
  ```
  $ cat test.rb
  {a: 1}.fetch(:b)
  $ ruby test.rb
  # Ruby 2.4:
  test.rb:1:in `fetch': key not found: :b (KeyError)
    from test.rb:1:in `<main>'

  # Ruby 2.5:
  Traceback (most recent call last):
    1: from test.rb:1:in `<main>'
  test.rb:1:in `fetch': key not found: :b (KeyError)
  ```
* **Follow-up:** Feature is tagged "experimental" in Ruby 2.5, somewhat [limited](2.6.html#exception-output-tweaking) in 2.6, and [completely reverted](3.0.html#exception-output-order-is-changed----again) in 3.0.

#### `Exception#full_message`[](#exceptionfull_message)

`Exception#full_message` provides the same string Ruby will print if exception is not caught.

* **Discussion:** <a class="tracker feature" href="https://bugs.ruby-lang.org/issues/14141">Feature #14141</a>
* **Documentation:** <a class="ruby-doc" href="https://ruby-doc.org/core-2.5.0/Exception.html#method-i-full_message"><code>Exception#full_message</code></a>
* **Code:**
  ```ruby
  # test.rb
  begin
    {a: 1}.fetch(:b)
  rescue => e
    puts e.full_message
  end
  # `ruby test.rb` prints:
  #   Traceback (most recent call last):
  #     1: from test.rb:2:in `<main>'
  #   test.rb:2:in `fetch': key not found: :b (KeyError)
  #
  # `ruby test.rb 2>err.log` prints:
  #   test.rb:2:in `fetch': key not found: :b (KeyError)
  #     from test.rb:2:in `<main>
  ```
* **Notice:** The format is different depending of whether **`STDERR`** is TTY or not (even if you printing the returned string to `STDOUT` or just storing it in the variable)
* **Follow-up:** Ruby 2.6 added [options for `#full_message`](https://rubyreferences.github.io/rubychanges/2.6.html#exceptionfull_message-options) for enforcing desired format regardless of whether `STDERR` is TTY

#### `KeyError#receiver` and `#key`[](#keyerrorreceiver-and-key)

* **Discussion:** <a class="tracker feature" href="https://bugs.ruby-lang.org/issues/12063">Feature #12063</a>
* **Documentation:** <a class="ruby-doc" href="https://ruby-doc.org/core-2.5.0/KeyError.html"><code>KeyError</code></a>
* **Code:**
  ```ruby
  begin
    {a: 1}.fetch(:b)
  rescue => e
    p e.receiver # => {:a=>1}
    p e.key      # => :b
  end
  ```
* **Follow-up:** Since Ruby 2.6, `receiver:` and `key:` [could also be passed](https://rubyreferences.github.io/rubychanges/2.6.html#new-arguments-receiver-and-key) when creating `KeyError` in your code (as of Ruby 2.5, only C code could've create "proper" `KeyError`)

#### New class: `FrozenError`[](#new-class-frozenerror)

* **Discussion:** <a class="tracker feature" href="https://bugs.ruby-lang.org/issues/13224">Feature #13224</a>
* **Documentation:** <a class="ruby-doc" href="https://ruby-doc.org/core-2.5.0/FrozenError.html"><code>FrozenError</code></a>
* **Code:**
  ```ruby
  a = [1, 2, 3].freeze
  a << 4
  # Ruby 2.4: RuntimeError: can't modify frozen Array
  # Ruby 2.5: FrozenError (can't modify frozen Array)
  ```

#### Don't hide coercion errors[](#dont-hide-coercion-errors)

Previously, some `Numeric` and `Range` methods caught errors when the provided values were incompatible, and thrown their own. This was hard to debug, so now the original error is thrown instead.

* **Discussion:** <a class="tracker feature" href="https://bugs.ruby-lang.org/issues/7688">Feature #7688</a>
* **Affected methods:** `Numeric#step`, `#<`, `#>`, `#>=`, `#<=` (when `other.coerce` method fails), `Range#initialize` (when `<=>` method of range's ends fail)
* **Code:**
  ```ruby
  class MyString
    def initialize(str)
      @str = str
    end

    def <=>(other)
      # This will fail: we forgot to define attr_reader :str
      str <=> other.str
    end

    include Comparable
  end

  'bar'..MyString.new('foo')
  # Ruby 2.4: test.rb:13:in `<main>': bad value for range (ArgumentError)
  # Ruby 2.5: test.rb:7:in `<=>': undefined local variable or method `str' for #<MyString:0x0000556d024e7b08 @str="bar"> (NameError)
  ```

### Filesystem and IO[](#filesystem-and-io)

#### `IO#pread` and `#pwrite`[](#iopread-and-pwrite)

Two atomic methods utilize corresponding system calls to read or write file at specified offset, instead of `#seek` + `#read`/`#write`

* **Reason:**
* **Discussion:** <a class="tracker feature" href="https://bugs.ruby-lang.org/issues/4532">Feature #4532</a>
* **Documentation:** <a class="ruby-doc" href="https://ruby-doc.org/core-2.5.0/IO.html#method-i-pread"><code>IO#pread</code></a>, <a class="ruby-doc" href="https://ruby-doc.org/core-2.5.0/IO.html#method-i-pwrite"><code>IO#pwrite</code></a>
* **Code:**
  ```ruby
  File.write('tmp.txt', 'test me please')
  f = File.open('tmp.txt')
  f.pread(2, 5) # => "me"
  f.pos         # => 0, unchanged
  ```

#### `IO#write` accepts multiple arguments[](#iowrite-accepts-multiple-arguments)

* **Discussion:** <a class="tracker feature" href="https://bugs.ruby-lang.org/issues/9323">Feature #9323</a>
* **Documentation:** <a class="ruby-doc" href="https://ruby-doc.org/core-2.5.0/IO.html#method-i-write"><code>IO#write</code></a>
* **Code:**
  ```ruby
  $stdout.write('test ', 'me ', 'please')
  # Prints: test me please
  ```
* **Notice:** Standard library classes that inherit or mimic IO interface were also updated: <a class="ruby-doc" href="https://ruby-doc.org/stdlib-2.5.0/libdoc/stringio/rdoc/StringIO.html#method-i-write"><code>StringIO#write</code></a>, <a class="ruby-doc" href="https://ruby-doc.org/stdlib-2.5.0/libdoc/zlib/rdoc/Zlib/GzipWriter.html#method-i-write"><code>Zlib::GZipWriter#write</code></a>

#### `File.open` better supports `newline:` option[](#fileopen-better-supports-newline-option)

Somewhat obscurely present `IO.new`/`IO.read`/`File.open` and similar methods options for converting newlines now properly switches file reading to text mode.

* **Discussion:** <a class="tracker bug" href="https://bugs.ruby-lang.org/issues/13350">Bug #13350</a>
* **Documentation:** — (it is _almost_ documented by referring from <a class="ruby-doc" href="https://ruby-doc.org/core-2.5.0/IO.html#method-c-new"><code>IO.new</code></a> docs to <a class="ruby-doc" href="https://ruby-doc.org/core-2.5.0/String.html#method-i-encode"><code>String#encode</code></a> docs, where at least longer form `newline_universal: true` is mentioned)
* **Code:**
  ```ruby
  File.write("crlf.txt", "a\nb\nc", newline: :crlf) # option to convert newlines to Windows format
  File.binread('crlf.txt')
  # => "a\r\nb\r\nc" -- what it really wrote
  File.read('crlf.txt', newline: :universal)
  # Ruby 2.4:
  #   => "a\r\nb\r\nc" -- newline option is ignored unless encodings passed too
  # Ruby 2.5:
  #   => "a\nb\nc" -- newline option is respected, file read in text mode & newlines converted
  ```

#### `File#path` raises when opened with `File::Constants::TMPFILE` option.[](#filepath-raises-when-opened-with-fileconstantstmpfile-option)

The constant `TMPFILE` is <a class="ruby-doc" href="https://ruby-doc.org/core-2.5.0/File/Constants.html#TMPFILE"><code>documented</code></a> as "Create an unnamed temporary file", but before Ruby 2.5, it was still possible to request this file's path.

* **Discussion:** <a class="tracker feature" href="https://bugs.ruby-lang.org/issues/13568">Feature #13568</a>
* **Documentation:** <a class="ruby-doc" href="https://ruby-doc.org/core-2.5.0/File.html#method-c-path"><code>File#path</code></a>
* **Code:**
  ```ruby
  f = File.open('/tmp', File::RDWR | File::TMPFILE)
  f.path
  # Ruby 2.4: => "/tmp" -- what???
  # Ruby 2.5: IOError (File is unnamed (TMPFILE?))
  ```
* **Follow-up:** Ruby 3.2 [introduced](3.2.html#iopath) a concept of generic `IO#path` which is allowed to be `nil` when the path is unknown, so the behavior changed:
  ```ruby
  f = File.open('/tmp', File::RDWR | File::TMPFILE)
  f.path #=> nil
  ```

#### `File.lutime`[](#filelutime)

Same as `utime`, but for symlinks sets modification time of link itself, not the referred object

* **Discussion:** <a class="tracker feature" href="https://bugs.ruby-lang.org/issues/4052">Feature #4052</a>
* **Documentation:** <a class="ruby-doc" href="https://ruby-doc.org/core-2.5.0/File.html#method-c-lutime"><code>File.lutime</code></a>

####  `Dir.children` and `.each_child`[](#dirchildren-and-each_child)

* **Discussion:** <a class="tracker feature" href="https://bugs.ruby-lang.org/issues/11302">Feature #11302</a>
* **Documentation:** <a class="ruby-doc" href="https://ruby-doc.org/core-2.5.0/Dir.html#method-c-children"><code>Dir.children</code></a>, <a class="ruby-doc" href="https://ruby-doc.org/core-2.5.0/Dir.html#method-c-each_child"><code>Dir.each_child</code></a>
* **Code:**
  ```ruby
  Dir.children('.')
  # => ["_layouts", ".gitignore", "README.md", "_data", "_src", "_site", "Contributing.md", "_config.yml", "2.6.md", "js", "404.html", ".git", "images", "Gemfile", "css", "Gemfile.lock"]
  ```
* **Follow-up:** Ruby 2.6 added [instance methods](https://rubyreferences.github.io/rubychanges/2.6.html#direach_child-and-dirchildren) with same behavior

#### `Dir.glob`: `base:` argument[](#dirglob-base-argument)

* **Discussion:** <a class="tracker feature" href="https://bugs.ruby-lang.org/issues/13056">Feature #13056</a>
* **Documentation:** <a class="ruby-doc" href="https://ruby-doc.org/core-2.5.0/Dir.html#method-c-glob"><code>Dir.glob</code></a>
* **Code:**
  ```ruby
  Dir.glob('*')
  # => ["_layouts", "README.md", "err.log", "_data", "_src", "_site", "Contributing.md", "_config.yml", "2.6.md", "js", "404.html", "images", "Gemfile", "css", "Gemfile.lock"]
  Dir.glob('*', base: '_src')
  # => ["2.5.md", "script", "tmp", "2.6.md"]
  ```

### `Process.last_status` as an alias of `$?`[](#processlast_status-as-an-alias-of-)

* **Discussion:** <a class="tracker feature" href="https://bugs.ruby-lang.org/issues/14043">Feature #14043</a>
* **Documentation:** <a class="ruby-doc" href="https://ruby-doc.org/core-2.5.0/Process.html#method-c-last_status"><code>Process.last_status</code></a>

### `Thread#fetch`[](#threadfetch)

Much akin to `Hash#fetch`, allows to fetch thread-local variable or provide default value

* **Discussion:** <a class="tracker feature" href="https://bugs.ruby-lang.org/issues/13009">Feature #13009</a>
* **Documentation:** <a class="ruby-doc" href="https://ruby-doc.org/core-2.6/Thread.html#method-i-fetch"><code>Thread#fetch</code></a> (feature is present since Ruby 2.5, but explanations added later)
* **Code:**
  ```ruby
  threads = [Thread.new { Thread.current[:name] = "first" }, Thread.new {}]
  threads.each(&:join)
  threads.map { |t| t.fetch(:name, '<not set>') }
  # => ["first", "<not set>"]
  threads.last.fetch(:name)
  # KeyError (key not found: :name)
  threads.last.fetch(:name) { |key| "#{key} not set at #{Time.now}" }
  # => "name not set at 2019-05-30 16:50:44 +0300"
  ```

### `RubyVM::InstructionSequence` new methods[](#rubyvminstructionsequence-new-methods)

* **Documentation:** <a class="ruby-doc" href="https://ruby-doc.org/core-2.5.0/RubyVM/InstructionSequence.html#method-i-each_child"><code>RubyVM::InstructionSequence#each_child</code></a>, <a class="ruby-doc" href="https://ruby-doc.org/core-2.5.0/RubyVM/InstructionSequence.html#method-i-trace_points"><code>RubyVM::InstructionSequence#trace_points</code></a>

### Misc[](#misc)

* `Random.raw_seed` renamed to <a class="ruby-doc" href="https://ruby-doc.org/core-2.5.0/Random.html#method-c-urandom"><code>Random.urandom</code></a>. Discussion: <a class="tracker bug" href="https://bugs.ruby-lang.org/issues/9569">Bug #9569</a>.
* `Data` is deprecated. It was a base class for C extensions, and it's not necessary to expose in Ruby level. <a class="tracker feature" href="https://bugs.ruby-lang.org/issues/3072">Feature #3072</a>. **Follow-up:** Fully removed in 3.0; in 3.2, a new class with the same name but different meaning was [reintroduced](3.2.html#data-new-immutable-value-object-class).

## Standard library[](#standard-library)

### Data types[](#data-types)

* `BigDecimal` is now behaving more consistently with core numeric classes: `BigDecimal.new` is deprecated in favor of <a class="ruby-doc" href="https://ruby-doc.org/stdlib-2.5.0/libdoc/bigdecimal/rdoc/Kernel.html#method-i-BigDecimal"><code>Kernel#BigDecimal()</code></a>, and `#clone` and `#dup` methods return the same instance.
* `Matrix`:
  * <a class="ruby-doc" href="https://ruby-doc.org/stdlib-2.5.0/libdoc/matrix/rdoc/Matrix.html#method-c-combine"><code>Matrix.combine</code></a> and <a class="ruby-doc" href="https://ruby-doc.org/stdlib-2.5.0/libdoc/matrix/rdoc/Matrix.html#method-i-combine"><code>Matrix#combine</code></a>. Discussion: <a class="tracker feature" href="https://bugs.ruby-lang.org/issues/10903">Feature #10903</a>
  * <a class="ruby-doc" href="https://ruby-doc.org/stdlib-2.5.0/libdoc/matrix/rdoc/Matrix.html#method-i-hadamard_product"><code>Matrix#hadamard_product</code></a> (aliased as <a class="ruby-doc" href="https://ruby-doc.org/stdlib-2.5.0/libdoc/matrix/rdoc/Matrix.html#method-i-entrywise_product"><code>#entrywise_product</code></a>)

#### `Set`[](#set)

* <a class="ruby-doc" href="https://ruby-doc.org/stdlib-2.5.0/libdoc/set/rdoc/Set.html#method-i-to_s"><code>Set#to_s</code></a> is an alias of `#inspect`. Discussion: <a class="tracker feature" href="https://bugs.ruby-lang.org/issues/13676">Feature #13676</a>
  ```ruby
  puts Set[:foo, :bar]
  # Ruby 2.4: #<Set:0x00005614c9069e68>
  # Ruby 2.5: #<Set: {:foo, :bar}>
  ```
* <a class="ruby-doc" href="https://ruby-doc.org/stdlib-2.5.0/libdoc/set/rdoc/Set.html#method-i-3D-3D-3D"><code>Set#===</code></a> is an alias of `#include?`. Discussion:  <a class="tracker feature" href="https://bugs.ruby-lang.org/issues/13801">Feature #13801</a>. This allows the use Sets in `case` and `grep`:
  ```ruby
  case RUBY_VERSION
  when Set['2.5.0', '2.5.1', '2.5.2', '2.5.3']
    # ...
  end

  [:open, :work, :pause, :close, :open, :break].grep(Set[:open, :close])
  # => [:open, :close, :open]
  ```
* <a class="ruby-doc" href="https://ruby-doc.org/stdlib-2.5.0/libdoc/set/rdoc/Set.html#method-i-reset"><code>Set#reset</code></a> (discussion: <a class="tracker feature" href="https://bugs.ruby-lang.org/issues/6589">Feature #6589</a>) allows to ensure set's internal state consistency after modifying a mutable element of the set:
  ```ruby
  processes = Set[[:open], [:work]]
  processes.first << :close
  processes
  # => #<Set: {[:open, :close], [:work]}>
  processes.include?([:open, :close]) # => false
  processes.reset
  processes.include?([:open, :close]) # => true
  ```

### Text processing[](#text-processing)

* `StringScanner`: methods <a class="ruby-doc" href="https://ruby-doc.org/stdlib-2.5.0/libdoc/strscan/rdoc/StringScanner.html#method-i-size"><code>#size</code></a>, <a class="ruby-doc" href="https://ruby-doc.org/stdlib-2.5.0/libdoc/strscan/rdoc/StringScanner.html#method-i-captures"><code>#captures</code></a>, <a class="ruby-doc" href="https://ruby-doc.org/stdlib-2.5.0/libdoc/strscan/rdoc/StringScanner.html#method-i-values_at"><code>#values_at</code></a> to introspect latest match, consistent with those of <a class="ruby-doc" href="https://ruby-doc.org/core-2.5.0/MatchData.html"><code>MatchData</code></a>. Discussion: <a class="tracker feature" href="https://bugs.ruby-lang.org/issues/836">Feature #836</a>
  ```ruby
  scanner = StringScanner.new('y = x^2')
  scanner.scan(/(\w+)\s*=\s*/)
  # => "y = "
  scanner.size            # number of captures, including entire matched string
  # => 2
  scanner.values_at(0, 1) # 0 is "entire matched sequence", 1 is "first capture"
  # => ["y = ", "y"]
  scanner.captures        # only captures
  # => ["y"]
  ```
* <a class="ruby-doc" href="https://ruby-doc.org/stdlib-2.5.0/libdoc/erb/rdoc/ERB.html#method-i-result_with_hash"><code>ERB#result_with_hash</code></a> added to allow passing local variables via hash instead of `Binding` object. Discussion: <a class="tracker feature" href="https://bugs.ruby-lang.org/issues/8631">Feature #8631</a>
  ```ruby
  require 'erb'
  ERB.new('x = <%= x %>').result_with_hash(x: 11)
  # => "x = 11"
  ```

### Network and Web[](#network-and-web)

* `open-uri`: `URI.open` method defined as an alias to open-uri's `Kernel#open`. open-uri's `Kernel#open` will be deprecated in future. Follow-up: Deprecation of `Kernel#open` and docs for <a class="ruby-doc" href="https://ruby-doc.org/stdlib-2.7.0/libdoc/open-uri/rdoc/URI.html#method-c-open"><code>URI.open</code></a> [were added](2.7.html#network-and-web) in Ruby 2.7. `Kernel#open` redefinition [was fully removed](3.0.html#network) in 3.0.

#### `Net::HTTP`[](#nethttp)

* New `no_proxy` argument to <a class="ruby-doc" href="https://ruby-doc.org/stdlib-2.5.0/libdoc/net/http/rdoc/Net/HTTP.html#method-c-new"><code>Net::HTTP.new</code></a>, allows to specify list of hosts that should be contacted directly, bypassing the proxy. Discussion: <a class="tracker feature" href="https://bugs.ruby-lang.org/issues/11195">Feature #11195</a>
* <a class="ruby-doc" href="https://ruby-doc.org/stdlib-2.5.0/libdoc/net/http/rdoc/Net/HTTP.html#method-i-proxy_user"><code>Net::HTTP#proxy_user</code></a> and <a class="ruby-doc" href="https://ruby-doc.org/stdlib-2.5.0/libdoc/net/http/rdoc/Net/HTTP.html#method-i-proxy_pass"><code>Net::HTTP#proxy_pass</code></a> now can be fetched from `ENV['http_proxy']` if it contains them and if OS provides multiuser safe environment (read: Unix-based OS). <a class="tracker bug" href="https://bugs.ruby-lang.org/issues/12921">Bug #12921</a>:
  ```ruby
  ENV['http_proxy'] = 'https://me:secret@proxy.server'
  http = Net::HTTP.new('https://google.com')
  http.proxy_user
  # => "me"
  http.proxy_pass
  # => "secret"
  ```
* `Net::HTTP#min_version` and `Net::HTTP#max_version` allow to specify min/max SSL version to use. Docs of <a class="ruby-doc" href="https://ruby-doc.org/stdlib-2.5.0/libdoc/openssl/rdoc/OpenSSL/SSL/SSLContext.html#method-i-min_version-3D">methods with same names</a> in OpenSSL library provide some explanations. Discussion: <a class="tracker feature" href="https://bugs.ruby-lang.org/issues/9450">Feature #9450</a>
* `Net::HTTP::STATUS_CODES` constant (_no docs_) is added as HTTP Status Code Repository. Discussion: <a class="tracker misc" href="https://bugs.ruby-lang.org/issues/12935">Misc #12935</a>
  ```ruby
  require 'net/http/status'
  Net::HTTP::STATUS_CODES.fetch(417)
  # => "Expectation Failed"
  ```

#### WEBrick[](#webrick)

* Support for `Proc` (or any other callable) objects as body responses (_no docs_). Discussion: <a class="tracker feature" href="https://bugs.ruby-lang.org/issues/855">Feature #855</a>:
  ```ruby
  require 'webrick'

  server = WEBrick::HTTPServer.new :Port => 8000

  trap 'INT' do server.shutdown end

  server.mount_proc '/' do |req, res|
    res.chunked = true # or set headers['content-length'] explicitly
    res.body = proc { |out| out << Time.now.to_s }
  end

  server.start
  ```
* [Server Name Indication](https://en.wikipedia.org/wiki/Server_Name_Indication) support via `:ServerName` option for <a class="ruby-doc" href="https://ruby-doc.org/stdlib-2.5.0/libdoc/webrick/rdoc/WEBrick/HTTPServer.html#method-c-new"><code>Webrick::HTTPServer.new</code></a>. Discussion: <a class="tracker feature" href="https://bugs.ruby-lang.org/issues/13729">Feature #13729</a>

### Ruby development and introspection[](#ruby-development-and-introspection)

* Coverage: support branch coverage and method coverage measurement. Discussion: <a class="tracker feature" href="https://bugs.ruby-lang.org/issues/13901">Feature #13901</a>
* `Binding#irb` (which is available, if not documented, since Ruby 2.4, meaning "start IRB session in the current scope"), now automatically requires `irb`. Discussion: <a class="tracker bug" href="https://bugs.ruby-lang.org/issues/13099">Bug #13099</a>
* `RbConfig::LIMITS` is added to provide the limits of C types:
  ```ruby
  require 'rbconfig/sizeof'
  RbConfig::LIMITS['FIXNUM_MAX']
  # => 4611686018427387903
  ```
* <a class="ruby-doc" href="https://ruby-doc.org/stdlib-2.5.0/libdoc/ripper/rdoc/Ripper/Filter.html#method-i-state"><code>Ripper::Filter#state</code></a> to tell the state of scanner. Discussion <a class="tracker feature" href="https://bugs.ruby-lang.org/issues/13686">Feature #13686</a>

### Misc[](#misc)

* <a class="ruby-doc" href="https://ruby-doc.org/stdlib-2.5.0/libdoc/pathname/rdoc/Pathname.html#method-c-glob"><code>Pathname#glob</code></a> Discussion: <a class="tracker feature" href="https://bugs.ruby-lang.org/issues/7360">Feature #7360</a>:
  ```ruby
  require 'pathname'
  Pathname.new('.').glob('*.md')
  # => [#<Pathname:2.5.md>, #<Pathname:README.md>, #<Pathname:Contributing.md>, #<Pathname:2.6.md>]
  ```
* <a class="ruby-doc" href="https://ruby-doc.org/stdlib-2.5.0/libdoc/securerandom/rdoc/Random/Formatter.html#method-i-alphanumeric"><code>SecureRandom.alphanumeric</code></a>
* `mathn.rb` removed from stdlib. <a class="tracker feature" href="https://bugs.ruby-lang.org/issues/10169">Feature #10169</a>. It <a class="github" href="https://github.com/ruby/ruby/blob/ruby_2_4/lib/mathn.rb#L7">was</a> a library that patched some core classes to make Ruby's math more "natural" (by providing `Rational` and `Complex` operations where necessary), but it was decided that side-effects of its implicit inclusion were too sudden, and explicit usage of Rational and Complex became much more natural in later Ruby versions.

### Large updated libraries[](#large-updated-libraries)

* **Psych** 3.0.2: _No changelog available._
* **RDoc** 6.0.1: _<a class="github" href="https://github.com/ruby/rdoc/blob/master/History.rdoc">Changelog</a> is outdated._
* **Rubygems** 2.7.3: <a class="github" href="https://github.com/rubygems/rubygems/blob/9bde8cd05e9e2f4a73d6783f09896dd551b58ebe/History.txt">Changelog</a>
* **IPAddr** 1.2.1: <a class="github" href="https://github.com/ruby/ipaddr/blob/master/CHANGELOG.md#121">Changelog</a>
* **OpenSSL** 2.1: <a class="github" href="https://github.com/ruby/ruby/blob/trunk/ext/openssl/History.md#version-210">Changelog</a>.

### Libraries promoted to default gems[](#libraries-promoted-to-default-gems)

> **[stdgems.org](https://stdgems.org/)** project has a nice explanations of default and bundled gems concepts, as well as a list of currently gemified libraries.

"For the rest of us" this means libraries development extracted into separate GitHub repositories, and they are just packaged with main Ruby before release. It means you can do issue/PR to any of them independently, without going through more tough development process of the core Ruby.

Libraries extracted in 2.5:

* <a class="github" href="https://github.com/ruby/cmath">cmath</a>
* <a class="github" href="https://github.com/ruby/csv">csv</a>
* <a class="github" href="https://github.com/ruby/date">date</a>
* <a class="github" href="https://github.com/ruby/dbm">dbm</a>
* <a class="github" href="https://github.com/ruby/etc">etc</a>
* <a class="github" href="https://github.com/ruby/fcntl">fcntl</a>
* <a class="github" href="https://github.com/ruby/fiddle">fiddle</a>
* <a class="github" href="https://github.com/ruby/fileutils">fileutils</a>
* <a class="github" href="https://github.com/ruby/gdbm">gdbm</a>
* <a class="github" href="https://github.com/ruby/ipaddr">ipaddr</a>
* <a class="github" href="https://github.com/ruby/scanf">scanf</a>
* <a class="github" href="https://github.com/ruby/sdbm">sdbm</a>
* <a class="github" href="https://github.com/ruby/stringio">stringio</a>
* <a class="github" href="https://github.com/ruby/strscan">strscan</a>
* <a class="github" href="https://github.com/ruby/webrick">webrick</a>
* <a class="github" href="https://github.com/ruby/zlib">zlib</a>

**Follow-up:**
* [14 more libraries](2.6.html#libraries-promoted-to-default-gems) gemified in 2.6;
* [16 more libraries](2.7.html#libraries-promoted-to-default-gems) gemified in 2.7, and [6 just dropped](2.7.html#libraries-excluded-from-the-standard-library) from the standard library (including cmath);
* [34 (!) more libraries](3.0.html#libraries-promoted-to-default-gems) gemified in 3.0, and [3 more just dropped](3.0.html#libraries-promoted-to-default-gems) from the standard library (including sdbm and webrick).
